#include "main.h"
#include "hslee_delay.h"
#include "hslee_uart.h"
#include "hslee_i2c_slave.h"

#pragma config FPLLMUL  = MUL_20        // PLL Multiplier
#pragma config FPLLIDIV = DIV_5         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN   = OFF           // Watchdog Timer
#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
#pragma config DEBUG    = OFF           // Debugger Disabled for Starter Kit


volatile unsigned char dataRead = 0;

int main(void)
{
    //SYSTEMConfigPerformance(SYS_FREQ);
    
    SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // Explorer16 LEDs are on lower 8-bits of PORTA and to use all LEDs, JTAG port must be disabled.
    mJTAGPortEnable(DEBUG_JTAGPORT_OFF);

    Init_Uart1(115200);
    Init_Uart2(115200);

    Init_i2c_slave();
    INTEnableSystemMultiVectoredInt();   // Interrupt vectoring

    mPORTAClearBits(BIT_4 | BIT_5 | BIT_6 | BIT_7);
    mPORTASetPinsDigitalOut(BIT_4 | BIT_5 | BIT_6 | BIT_7);

    PORTAbits.RA4 = 0;
    PORTAbits.RA5 = 0;
    PORTAbits.RA6 = 0;
    PORTAbits.RA7 = 0;
    
    while(1)
    {
        LED1 = ~LED1;        LED2 = ~LED2;        LED3 = ~LED3;        LED4 = ~LED4;
        
        Delayms(500);
    }
    return 0;
}



unsigned char uart1_rx;
void __ISR(_UART1_VECTOR, ipl5) _uart1(void) 
{
	if(INTGetFlag(INT_SOURCE_UART_RX(UART1)))
	{
		INTClearFlag(INT_SOURCE_UART_RX(UART1));
		U1STAbits.OERR=0;
		uart1_rx = UARTGetDataByte(UART1);
		TxData1(uart1_rx);
	}	
}	


unsigned char uart2_rx;
void __ISR(_UART2_VECTOR, ipl3) _uart2(void) 	
{
    if(INTGetFlag(INT_SOURCE_UART_RX(UART2)))
	{
		INTClearFlag(INT_SOURCE_UART_RX(UART2));
		U1STAbits.OERR=0;
		uart2_rx = UARTGetDataByte(UART2);
		if(uart2_rx != 0x00)
			TxData1(uart2_rx);
	}	
}



void __ISR(_I2C_1_VECTOR, ipl3) _SlaveI2CHandler(void)
{
    unsigned char temp;
    static unsigned int dIndex;

    // check for MASTER and Bus events and respond accordingly
    if (IFS0bits.I2C1MIF == 1)
    {
        mI2C1MClearIntFlag();
        return;
    }
    if (IFS0bits.I2C1BIF == 1)
    {
        mI2C1BClearIntFlag();
        return;
    }

    // handle the incoming message
    if ((I2C1STATbits.R_W == 0) && (I2C1STATbits.D_A == 0))
    {
        // R/W bit = 0 --> indicates data transfer is input to slave
        // D/A bit = 0 --> indicates last byte was address

        // reset any state variables needed by a message sequence
        // perform a dummy read of the address
        temp = SlaveReadI2C1();

        // release the clock to restart I2C
        I2C1CONbits.SCLREL = 1; // release the clock

    }
    else if ((I2C1STATbits.R_W == 0) && (I2C1STATbits.D_A == 1))
    {
        // R/W bit = 0 --> indicates data transfer is input to slave
        // D/A bit = 1 --> indicates last byte was data

        // writing data to our module, just store it in adcSample
        dataRead = SlaveReadI2C1();

        if(dataRead == 0xAA)
        {
            PORTAbits.RA6 = ~PORTAbits.RA6;
            PORTAbits.RA7 = ~PORTAbits.RA7;
        }
        // release the clock to restart I2C
        I2C1CONbits.SCLREL = 1; // release clock stretch bit

    }
    else if ((I2C1STATbits.R_W == 1) && (I2C1STATbits.D_A == 0))
    {
        // R/W bit = 1 --> indicates data transfer is output from slave
        // D/A bit = 0 --> indicates last byte was address

        // read of the slave device, read the address
        temp = SlaveReadI2C1();
        dIndex = 0;
        SlaveWriteI2C1(dataRead);
    }
    else if ((I2C1STATbits.R_W == 1) && (I2C1STATbits.D_A == 1))
    {
        // R/W bit = 1 --> indicates data transfer is output from slave
        // D/A bit = 1 --> indicates last byte was data

        // output the data until the MASTER terminates the
        // transfer with a NACK, continuing reads return 0
        if (dIndex == 0)
        {
            SlaveWriteI2C1(dataRead);
            dIndex++;
        }
        else
            SlaveWriteI2C1(0);
    }
    // finally clear the slave interrupt flag
    mI2C1SClearIntFlag();
}


